@@ -1,37 +1,23 @@
-context("testServer")
-
 library(shiny)
 library(testthat)
-library(future)
+library(future, warn.conflicts = FALSE)
 library(promises)
 
-test_that("testServer passes dots", {
-  module <- function(id, someArg) {
-    expect_false(missing(someArg))
-    moduleServer(id, function(input, output, session) {
-      expect_equal(someArg, 123)
+test_that("handles observers", {
+  server <- function(input, output, session) {
+    rv <- reactiveValues(x = 0, y = 0)
+    observe({
+      rv$x <- input$x * 2
     })
-  }
-  testServer(module, {}, args = list(someArg = 123))
-})
-
-test_that("testServer handles observers", {
-  module <- function(id) {
-    moduleServer(id, function(input, output, session) {
-      rv <- reactiveValues(x = 0, y = 0)
-      observe({
-        rv$x <- input$x * 2
-      })
-      observe({
-        rv$y <- rv$x
-      })
-      output$txt <- renderText({
-        paste0("Value: ", rv$x)
-      })
+    observe({
+      rv$y <- rv$x
+    })
+    output$txt <- renderText({
+      paste0("Value: ", rv$x)
     })
   }
 
-  testServer(module, {
+  testServer(server, {
     session$setInputs(x=1)
     expect_equal(rv$y, 2)
     expect_equal(rv$x, 2)
@@ -45,9 +31,9 @@ test_that("testServer handles observers", {
 })
 
 test_that("inputs aren't directly assignable", {
-  module <- function(id) moduleServer(id, function(input, output, session) {})
+  server <- function(input, output, session) {}
 
-  testServer(module, {
+  testServer(server, {
     session$setInputs(x = 0)
     expect_error({ input$x <- 1 })
     expect_error({ input$y <- 1 })
@@ -55,12 +41,12 @@ test_that("inputs aren't directly assignable", {
 })
 
 test_that("setInputs dots are dynamic", {
-  module <- function(id) moduleServer(id, function(input, output, session) {})
+  server <- function(input, output, session) {}
 
   inputs_initial <- list(x=1, y=2)
   input_y <- "y"
 
-  testServer(module, {
+  testServer(server, {
     session$setInputs(!!!inputs_initial)
     expect_equal(input$x, 1)
     expect_equal(input$y, 2)
@@ -72,16 +58,14 @@ test_that("setInputs dots are dynamic", {
   })
 })
 
-test_that("testServer handles more complex expressions", {
-  module <- function(id) {
-    moduleServer(id, function(input, output, session){
-      output$txt <- renderText({
-        input$x
-      })
+test_that("handles more complex expressions", {
+  server  <- function(input, output, session) {
+    output$txt <- renderText({
+      input$x
     })
   }
 
-  testServer(module, {
+  testServer(server, {
     for (i in 1:5){
       session$setInputs(x=i)
       expect_equal(output$txt, as.character(i))
@@ -95,17 +79,15 @@ test_that("testServer handles more complex expressions", {
   })
 })
 
-test_that("testServer handles reactiveVal", {
-  module <- function(id) {
-    moduleServer(id, function(input, output, session) {
-      x <- reactiveVal(0)
-      observe({
-        x(input$y + input$z)
-      })
+test_that("handles reactiveVal", {
+  server <- function(input, output, session) {
+    x <- reactiveVal(0)
+    observe({
+      x(input$y + input$z)
     })
   }
 
-  testServer(module, {
+  testServer(server, {
     session$setInputs(y=1, z=2)
 
     expect_equal(x(), 3)
@@ -118,20 +100,18 @@ test_that("testServer handles reactiveVal", {
   })
 })
 
-test_that("testServer handles reactives with complex dependency tree", {
-  module <- function(id) {
-    moduleServer(id, function(input, output, session) {
-      x <- reactiveValues(x=1)
-      r <- reactive({
-        x$x + input$a + input$b
-      })
-      r2 <- reactive({
-        r() + input$c
-      })
+test_that("handles reactives with complex dependency tree", {
+  server <- function(input, output, session) {
+    x <- reactiveValues(x=1)
+    r <- reactive({
+      x$x + input$a + input$b
+    })
+    r2 <- reactive({
+      r() + input$c
     })
   }
 
-  testServer(module, {
+  testServer(server, {
     session$setInputs(a=1, b=2, c=3)
     expect_equal(r(), 4)
     expect_equal(r2(), 7)
@@ -150,20 +130,18 @@ test_that("testServer handles reactives with complex dependency tree", {
   })
 })
 
-test_that("testServer handles reactivePoll", {
-  module <- function(id) {
-    moduleServer(id, function(input, output, session) {
-      rv <- reactiveValues(x = 0)
-      rp <- reactivePoll(50, session, function(){ rnorm(1) }, function(){
-        isolate(rv$x <- rv$x + 1)
-        rnorm(1)
-      })
-
-      observe({rp()})
+test_that("handles reactivePoll", {
+  server <- function(input, output, session) {
+    rv <- reactiveValues(x = 0)
+    rp <- reactivePoll(50, session, function(){ rnorm(1) }, function(){
+      isolate(rv$x <- rv$x + 1)
+      rnorm(1)
     })
+
+    observe({rp()})
   }
 
-  testServer(module, {
+  testServer(server, {
     session$flushReact()
     expect_equal(rv$x, 1)
 
@@ -175,20 +153,18 @@ test_that("testServer handles reactivePoll", {
   })
 })
 
-test_that("testServer handles reactiveTimer", {
-  module <- function(id) {
-    moduleServer(id, function(input, output, session) {
-      rv <- reactiveValues(x = 0)
+test_that("handles reactiveTimer", {
+  server <- function(input, output, session) {
+    rv <- reactiveValues(x = 0)
 
-      rp <- reactiveTimer(50)
-      observe({
-        rp()
-        isolate(rv$x <- rv$x + 1)
-      })
+    rp <- reactiveTimer(50)
+    observe({
+      rp()
+      isolate(rv$x <- rv$x + 1)
     })
   }
 
-  testServer(module, {
+  testServer(server, {
     session$flushReact()
     expect_equal(rv$x, 1)
 
@@ -198,29 +174,27 @@ test_that("testServer handles reactiveTimer", {
   })
 })
 
-test_that("testServer handles debounce/throttle", {
-  module <- function(id) {
-    moduleServer(id, function(input, output, session) {
-      rv <- reactiveValues(t = 0, d = 0)
-      react <- reactive({
-        input$y
-      })
-      rt <- throttle(react, 100)
-      rd <- debounce(react, 100)
+test_that("handles debounce/throttle", {
+  server <- function(input, output, session) {
+    rv <- reactiveValues(t = 0, d = 0)
+    react <- reactive({
+      input$y
+    })
+    rt <- throttle(react, 100)
+    rd <- debounce(react, 100)
 
-      observe({
-        rt() # Invalidate this block on the timer
-        isolate(rv$t <- rv$t + 1)
-      })
+    observe({
+      rt() # Invalidate this block on the timer
+      isolate(rv$t <- rv$t + 1)
+    })
 
-      observe({
-        rd()
-        isolate(rv$d <- rv$d + 1)
-      })
+    observe({
+      rd()
+      isolate(rv$d <- rv$d + 1)
     })
   }
 
-  testServer(module, {
+  testServer(server, {
     session$setInputs(y = TRUE)
     expect_equal(rv$d, 1)
     for (i in 2:5){
@@ -240,7 +214,7 @@ test_that("testServer handles debounce/throttle", {
   })
 })
 
-test_that("testServer wraps output in an observer", {
+test_that("wraps output in an observer", {
   testthat::skip("I'm not sure of a great way to test this without timers.")
   # And honestly it's so foundational in what we're doing now that it might not be necessary to test?
 
@@ -281,26 +255,24 @@ test_that("testServer wraps output in an observer", {
   #  - plots and such?
 })
 
-test_that("testServer works with async", {
-  module <- function(id) {
-    moduleServer(id, function(input, output, session) {
-      output$txt <- renderText({
-        val <- input$x
-        future({ val })
-      })
+test_that("works with async", {
+  server <- function(input, output, session) {
+    output$txt <- renderText({
+      val <- input$x
+      future({ val })
+    })
 
-      output$error <- renderText({
-        future({ stop("error here") })
-      })
+    output$error <- renderText({
+      future({ stop("error here") })
+    })
 
-      output$sync <- renderText({
-        # No promises here
-        "abc"
-      })
+    output$sync <- renderText({
+      # No promises here
+      "abc"
     })
   }
 
-  testServer(module, {
+  testServer(server, {
     session$setInputs(x=1)
     expect_equal(output$txt, "1")
     expect_equal(output$sync, "abc")
@@ -317,26 +289,24 @@ test_that("testServer works with async", {
   })
 })
 
-test_that("testModule works with multiple promises in parallel", {
-  module <- function(id) {
-    moduleServer(id, function(input, output, session) {
-      output$txt1 <- renderText({
-        future({
-          Sys.sleep(1)
-          1
-        })
+test_that("works with multiple promises in parallel", {
+  server <- function(input, output, session) {
+    output$txt1 <- renderText({
+      future({
+        Sys.sleep(1)
+        1
       })
+    })
 
-      output$txt2 <- renderText({
-        future({
-          Sys.sleep(1)
-          2
-        })
+    output$txt2 <- renderText({
+      future({
+        Sys.sleep(1)
+        2
       })
     })
   }
 
-  testServer(module, {
+  testServer(server, {
     # As we enter this test code, the promises will still be running in the background.
     # We'll need to give them ~2s (plus overhead) to complete
     startMS <- as.numeric(Sys.time()) * 1000
@@ -355,71 +325,39 @@ test_that("testModule works with multiple promises in parallel", {
   })
 })
 
-test_that("testModule handles async errors", {
-  module <- function(id) {
-    moduleServer(id, function(input, output, session, arg1, arg2){
-      output$err <- renderText({
-        future({ "my error"}) %...>%
-          stop() %...>%
-          print() # Extra steps after the error
-      })
+test_that("handles async errors", {
+  server <- function(input, output, session, arg1, arg2){
+    output$err <- renderText({
+      future({ "my error"}) %...>%
+        stop() %...>%
+        print() # Extra steps after the error
+    })
 
-      output$safe <- renderText({
-        future({ safeError("my safe error") }) %...>%
-          stop()
-      })
+    output$safe <- renderText({
+      future({ safeError("my safe error") }) %...>%
+        stop()
     })
   }
 
-  testServer(module, {
+  testServer(server, {
     expect_error(output$err, "my error")
     # TODO: helper for safe errors so users don't have to learn "shiny.custom.error"?
     expect_error(output$safe, "my safe error", class="shiny.custom.error")
   })
 })
 
-test_that("testServer handles modules with additional arguments", {
-  module <- function(id, arg1, arg2) {
-    moduleServer(id, function(input, output, session){
-      output$txt1 <- renderText({
-        arg1
-      })
-
-      output$txt2 <- renderText({
-        arg2
-      })
-
-      output$inp <- renderText({
-        input$x
-      })
-    })
-  }
-
-  testServer(module, {
-    expect_equal(output$txt1, "val1")
-    expect_equal(output$txt2, "val2")
-  }, list(arg1="val1", arg2="val2"))
-})
-
-test_that("testServer captures htmlwidgets", {
+test_that("captures htmlwidgets", {
   # TODO: use a simple built-in htmlwidget instead of something complex like dygraph
-  if (!requireNamespace("dygraphs")){
-    testthat::skip("dygraphs not available to test htmlwidgets")
-  }
+  skip_if_not_installed("dygraphs")
+  skip_if_not_installed("jsonlite")
 
-  if (!requireNamespace("jsonlite")){
-    testthat::skip("jsonlite not available to test htmlwidgets")
-  }
-
-  module <- function(id) {
-    moduleServer(id, function(input, output, session){
-      output$dy <- dygraphs::renderDygraph({
-        dygraphs::dygraph(data.frame(outcome=0:5, year=2000:2005))
-      })
+  server <- function(input, output, session){
+    output$dy <- dygraphs::renderDygraph({
+      dygraphs::dygraph(data.frame(outcome=0:5, year=2000:2005))
     })
   }
 
-  testServer(module, {
+  testServer(server, {
     # Really, this test should be specific to each htmlwidget. Here, we don't want to bind ourselves
     # to the current JSON structure of dygraphs, so we'll just check one element to see that the raw
     # JSON was exposed and is accessible in tests.
@@ -429,35 +367,31 @@ test_that("testServer captures htmlwidgets", {
   })
 })
 
-test_that("testServer captures renderUI", {
-  module <- function(id) {
-    moduleServer(id, function(input, output, session){
-      output$ui <- renderUI({
-        tags$a(href="https://rstudio.com", "hello!")
-      })
+test_that("captures renderUI", {
+  server <- function(input, output, session){
+    output$ui <- renderUI({
+      tags$a(href="https://rstudio.com", "hello!")
     })
   }
 
-  testServer(module, {
+  testServer(server, {
     expect_equal(output$ui$deps, list())
     expect_equal(as.character(output$ui$html), "<a href=\"https://rstudio.com\">hello!</a>")
   })
 })
 
-test_that("testServer captures base graphics outputs", {
-  module <- function(id) {
-    moduleServer(id, function(input, output, session){
-      output$fixed <- renderPlot({
-        plot(1,1)
-      }, width=300, height=350)
+test_that("captures base graphics outputs", {
+  server <- function(input, output, session){
+    output$fixed <- renderPlot({
+      plot(1,1)
+    }, width=300, height=350)
 
-      output$dynamic <- renderPlot({
-        plot(1,1)
-      })
+    output$dynamic <- renderPlot({
+      plot(1,1)
     })
   }
 
-  testServer(module, {
+  testServer(server, {
     # We aren't yet able to create reproducible graphics, so this test is intentionally pretty
     # limited.
     expect_equal(output$fixed$width, 300)
@@ -474,24 +408,20 @@ test_that("testServer captures base graphics outputs", {
   })
 })
 
-test_that("testServer captures ggplot2 outputs", {
-  if (!requireNamespace("ggplot2")){
-    testthat::skip("ggplot2 not available")
-  }
+test_that("captures ggplot2 outputs", {
+  skip_if_not_installed("ggplot2")
 
-  module <- function(id) {
-    moduleServer(id, function(input, output, session){
-      output$fixed <- renderPlot({
-        ggplot2::qplot(iris$Sepal.Length, iris$Sepal.Width)
-      }, width=300, height=350)
+  server <- function(input, output, session){
+    output$fixed <- renderPlot({
+      ggplot2::qplot(iris$Sepal.Length, iris$Sepal.Width)
+    }, width=300, height=350)
 
-      output$dynamic <- renderPlot({
-        ggplot2::qplot(iris$Sepal.Length, iris$Sepal.Width)
-      })
+    output$dynamic <- renderPlot({
+      ggplot2::qplot(iris$Sepal.Length, iris$Sepal.Width)
     })
   }
 
-  testServer(module, {
+  testServer(server, {
     expect_equal(output$fixed$width, 300)
     expect_equal(output$fixed$height, 350)
     expect_match(output$fixed$src, "^data:image/png;base64,")
@@ -503,39 +433,19 @@ test_that("testServer captures ggplot2 outputs", {
   })
 })
 
-test_that("testServer exposes the returned value from the module", {
-  module <- function(id) {
-    moduleServer(id, function(input, output, session){
-      reactive({
-        return(input$a + input$b)
-      })
+
+test_that("handles synchronous errors", {
+  server <- function(input, output, session){
+    output$err <- renderText({
+      stop("my error")
+    })
+
+    output$safe <- renderText({
+      stop(safeError("my safe error"))
     })
   }
 
-  testServer(module, {
-    session$setInputs(a=1, b=2)
-    expect_equal(session$getReturned()(), 3)
-
-    # And retains reactivity
-    session$setInputs(a=2)
-    expect_equal(session$getReturned()(), 4)
-  })
-})
-
-test_that("testServer handles synchronous errors", {
-  module <- function(id) {
-    moduleServer(id, function(input, output, session, arg1, arg2){
-      output$err <- renderText({
-        stop("my error")
-      })
-
-      output$safe <- renderText({
-        stop(safeError("my safe error"))
-      })
-    })
-  }
-
-  testServer(module, {
+  testServer(server, {
     expect_error(output$err, "my error")
     # TODO: helper for safe errors so users don't have to learn "shiny.custom.error"?
     expect_error(output$safe, "my safe error", class="shiny.custom.error")
@@ -543,45 +453,26 @@ test_that("testServer handles synchronous errors", {
 })
 
 test_that("accessing a non-existent output gives an informative message", {
-  module <- function(id) {
-    moduleServer(id, function(input, output, session){})
-  }
+  server <- function(input, output, session){}
 
-  testServer(module, {
-    expect_error(output$dontexist, "hasn't been defined yet: output\\$server1-dontexist")
-  }, list(id = "server1"))
-
-  testServer(module, {
-    expect_error(output$dontexist, "hasn't been defined yet: output\\$.*-dontexist")
+  testServer(server, {
+    expect_error(output$dontexist, "hasn't been defined yet")
   })
 })
 
-test_that("assigning an output in a module function with a non-function errors", {
-  module <- function(id) {
-    moduleServer(id, function(input, output, session) {
-      output$someVar <- 123
-
+test_that("handles invalidateLater", {
+  server <- function(input, output, session) {
+    rv <- reactiveValues(x = 0)
+    observe({
+      isolate(rv$x <- rv$x + 1)
+      # We're only testing one invalidation
+      if (isolate(rv$x) <= 1){
+        invalidateLater(50)
+      }
     })
   }
 
-  expect_error(testServer(module, {}), "^Unexpected")
-})
-
-test_that("testServer handles invalidateLater", {
-  module <- function(id) {
-    moduleServer(id, function(input, output, session) {
-      rv <- reactiveValues(x = 0)
-      observe({
-        isolate(rv$x <- rv$x + 1)
-        # We're only testing one invalidation
-        if (isolate(rv$x) <= 1){
-          invalidateLater(50)
-        }
-      })
-    })
-  }
-
-  testServer(module, {
+  testServer(server, {
     session$flushReact()
     # Should have run once
     expect_equal(rv$x, 1)
@@ -596,11 +487,9 @@ test_that("testServer handles invalidateLater", {
 })
 
 test_that("session ended handlers work", {
-  module <- function(id) {
-    moduleServer(id, function(input, output, session){})
-  }
+  server <- function(input, output, session){}
 
-  testServer(module, {
+  testServer(server, {
     rv <- reactiveValues(closed = FALSE)
     session$onEnded(function(){
       rv$closed <- TRUE
@@ -619,23 +508,21 @@ test_that("session ended handlers work", {
 })
 
 test_that("session flush handlers work", {
-  module <- function(id) {
-    moduleServer(id, function(input, output, session) {
-      rv <- reactiveValues(x = 0, flushCounter = 0, flushedCounter = 0,
-                           flushOnceCounter = 0, flushedOnceCounter = 0)
+  server <- function(input, output, session) {
+    rv <- reactiveValues(x = 0, flushCounter = 0, flushedCounter = 0,
+                         flushOnceCounter = 0, flushedOnceCounter = 0)
 
-      onFlush(function(){rv$flushCounter <- rv$flushCounter + 1}, once=FALSE)
-      onFlushed(function(){rv$flushedCounter <- rv$flushedCounter + 1}, once=FALSE)
-      onFlushed(function(){rv$flushOnceCounter <- rv$flushOnceCounter + 1}, once=TRUE)
-      onFlushed(function(){rv$flushedOnceCounter <- rv$flushedOnceCounter + 1}, once=TRUE)
+    onFlush(function(){rv$flushCounter <- rv$flushCounter + 1}, once=FALSE)
+    onFlushed(function(){rv$flushedCounter <- rv$flushedCounter + 1}, once=FALSE)
+    onFlushed(function(){rv$flushOnceCounter <- rv$flushOnceCounter + 1}, once=TRUE)
+    onFlushed(function(){rv$flushedOnceCounter <- rv$flushedOnceCounter + 1}, once=TRUE)
 
-      observe({
-        rv$x <- input$x * 2
-      })
+    observe({
+      rv$x <- input$x * 2
     })
   }
 
-  testServer(module, {
+  testServer(server, {
     session$setInputs(x=1)
     expect_equal(rv$x, 2)
     # We're not concerned with the exact values here -- only that they increase
@@ -653,45 +540,22 @@ test_that("session flush handlers work", {
   })
 })
 
-test_that("module return value captured", {
-  module_implicit_return <- function(id) {
-    moduleServer(id, function(input, output, session) {
-      123
-    })
+test_that("can't test within test", {
+  server <- function(input, output, session) {
+    testServer()
   }
-
-  testServer(module_implicit_return, {
-    expect_equal(session$returned, 123)
-  })
-
-  module_early_returns <- function(id, n) {
-    retval <- NULL
-    moduleServer(id, function(input, output, session) {
-      if (n == 0) return(n)
-      if (n %% 2 == 0) {
-        retval <<- "even"
-      } else {
-        return(FALSE)
-      }
-      retval
-    })
-  }
-
-  testServer(module_early_returns, {
-    expect_equal(session$returned, 0)
-  }, args = list(n = 0))
-
-  testServer(module_early_returns, {
-    expect_equal(session$returned, FALSE)
-  }, args = list(n = 1))
-
-  testServer(module_early_returns, {
-    expect_equal(session$returned, "even")
-  }, args = list(n = 2))
+  expect_error(testServer(server, {}), "only within tests")
 })
 
-test_that("It's an error to pass arguments to a server", {
-  expect_error(testServer(test_path("..", "test-modules", "06_tabsets"), {}, args = list(an_arg = 123)))
+test_that("validates server function", {
+  server <- function(input, output, session) {}
+  expect_error(
+    testServer(server, {}, args = list(an_arg = 123)),
+    "Arguments were provided"
+  )
+
+  app <- shinyApp(fluidPage(), function(x, y, z) {})
+  expect_error(testServer(app, {}), "must declare")
 })
 
 # Provided an instance of an R6 object and its generator, returns a list with
@@ -700,7 +564,7 @@ test_that("It's an error to pass arguments to a server", {
 # bindings are considered `fields`.
 get_mocked_publics <- function(instance, generator) {
   publics <- ls(instance, all.names = TRUE)
-  actives <- names(generator$active) %OR% character(0)
+  actives <- names(generator$active) %||% character(0)
   # Active bindings are considered fields.
   methods_or_fields <- publics[!(publics %in% actives)]
   methods <- character(0)
@@ -732,22 +596,20 @@ test_that("downloadHandler() works", {
   data <- mtcars
   tmpd <- NULL
 
-  module <- function(id) {
-    moduleServer(id, function(input, output, session) {
-      filename <- reactive({
-        paste0(input$name, ".", input$extension)
-      })
-      output$downloadData <- downloadHandler(
-        filename = filename(),
-        content = function(file) {
-          tmpd <<- dirname(file)
-          saveRDS(data, file)
-        }
-      )
+  server <- function(input, output, session) {
+    filename <- reactive({
+      paste0(input$name, ".", input$extension)
     })
+    output$downloadData <- downloadHandler(
+      filename = filename(),
+      content = function(file) {
+        tmpd <<- dirname(file)
+        saveRDS(data, file)
+      }
+    )
   }
 
-  testServer(module, {
+  testServer(server, {
     session$setInputs(name = "mtcars", extension = "rds")
     f <- output$downloadData
     expect_equal(basename(f), "mtcars.rds")
@@ -761,24 +623,154 @@ test_that("downloadHandler() works", {
 test_that("getOutputInfo() returns current output name", {
   savedOutputInfo <- NULL
 
+  server <- function(input, output, session) {
+    output$txt <- renderText({
+      savedOutputInfo <<- getCurrentOutputInfo()
+      "some text"
+    })
+  }
+
+  testServer(server, {
+    expect_equal(savedOutputInfo, NULL)
+    # savedOutputInfo is not set until output$txt is accessed
+    expect_equal(output$txt, "some text")
+    expect_equal(savedOutputInfo, list(name = "txt"))
+    expect_equal(getCurrentOutputInfo(), NULL)
+  })
+})
+
+test_that("promise chains evaluate in correct order", {
+  messages <- list()
+  clearMessages <- function() {
+    messages <<- list()
+  }
+  pushMessage <- function(msg) {
+    messages <<- c(messages, msg)
+  }
+
+  server <- function(input, output, session) {
+    r1 <- reactive({
+      promise(function(resolve, reject) {
+        pushMessage("promise 1")
+        resolve(input$go)
+      })$then(function(value) {
+        pushMessage(paste("promise 1 then", value))
+        paste("r1", input$go)
+      })
+    })
+    r2 <- reactive({
+      promise(function(resolve, reject) {
+        pushMessage("promise 2")
+        resolve(input$go)
+      })$then(function(value) {
+        pushMessage(paste("promise 2 then", value))
+        paste("r2", input$go)
+      })
+    })
+    output$text1 <- renderText({
+      pushMessage("output$text1")
+      r1()
+    })
+    output$text2 <- renderText({
+      pushMessage("output$text2")
+      input$go
+      r2()
+    })
+  }
+
+  testServer(server, {
+    expect_length(messages, 0)
+    session$setInputs(go = 1)
+    expect_equal(output$text1, "r1 1")
+    expect_equal(output$text2, "r2 1")
+    expect_equal(messages, list(
+      "output$text1",
+      "promise 1",
+      "output$text2",
+      "promise 2",
+      "promise 1 then 1",
+      "promise 2 then 1"
+    ))
+    clearMessages()
+    session$setInputs(go = 2)
+    expect_equal(output$text1, "r1 2")
+    expect_equal(output$text2, "r2 2")
+    expect_equal(messages, list(
+      "output$text1",
+      "promise 1",
+      "output$text2",
+      "promise 2",
+      "promise 1 then 2",
+      "promise 2 then 2"
+    ))
+  })
+})
+
+# Modules specific behaviour  ------------------------------------------------
+
+test_that("exposes the returned value from the module", {
   module <- function(id) {
-    moduleServer(id, function(input, output, session) {
-      output$txt <- renderText({
-        savedOutputInfo <<- getCurrentOutputInfo()
-        "some text"
+    moduleServer(id, function(input, output, session){
+      reactive({
+        return(input$a + input$b)
       })
     })
   }
 
   testServer(module, {
-    expect_equal(savedOutputInfo, NULL)
-    # savedOutputInfo is not set until output$txt is accessed
-    expect_equal(output$txt, "some text")
-    expect_equal(savedOutputInfo, list(name = session$ns("txt")))
-    expect_equal(getCurrentOutputInfo(), NULL)
+    session$setInputs(a=1, b=2)
+    expect_equal(session$getReturned()(), 3)
+
+    # And retains reactivity
+    session$setInputs(a=2)
+    expect_equal(session$getReturned()(), 4)
   })
 })
 
+test_that("passes dots", {
+  module <- function(id, someArg) {
+    expect_false(missing(someArg))
+    moduleServer(id, server <- function(input, output, session) {
+      expect_equal(someArg, 123)
+    })
+  }
+  testServer(module, {}, args = list(someArg = 123))
+})
+
+test_that("handles modules with additional arguments", {
+  module <- function(id, arg1, arg2) {
+    moduleServer(id, function(input, output, session){
+      output$txt1 <- renderText({
+        arg1
+      })
+
+      output$txt2 <- renderText({
+        arg2
+      })
+
+      output$inp <- renderText({
+        input$x
+      })
+    })
+  }
+
+  testServer(module, {
+    expect_equal(output$txt1, "val1")
+    expect_equal(output$txt2, "val2")
+  }, list(arg1="val1", arg2="val2"))
+})
+
+
+test_that("assigning an output in a module function with a non-function errors", {
+  module <- function(id) {
+    moduleServer(id, function(input, output, session) {
+      output$someVar <- 123
+    })
+  }
+
+  expect_error(testServer(module, {}), "^Unexpected")
+})
+
 test_that("renderCachedPlot with cache = app and cache = session works", {
   module <- function(id, cache, callback) {
     moduleServer(id, function(input, output, session) {
@@ -819,71 +811,11 @@ test_that("renderCachedPlot with cache = app and cache = session works", {
 })
 
 
-test_that("promise chains evaluate in correct order", {
-  messages <- list()
-  clearMessages <- function() {
-    messages <<- list()
-  }
-  pushMessage <- function(msg) {
-    messages <<- c(messages, msg)
-  }
+# Helpers -----------------------------------------------------------------
 
-  module <- function(id) {
-    moduleServer(id, function(input, output, session) {
-      r1 <- reactive({
-        promise(function(resolve, reject) {
-          pushMessage("promise 1")
-          resolve(input$go)
-        })$then(function(value) {
-          pushMessage(paste("promise 1 then", value))
-          paste("r1", input$go)
-        })
-      })
-      r2 <- reactive({
-        promise(function(resolve, reject) {
-          pushMessage("promise 2")
-          resolve(input$go)
-        })$then(function(value) {
-          pushMessage(paste("promise 2 then", value))
-          paste("r2", input$go)
-        })
-      })
-      output$text1 <- renderText({
-        pushMessage("output$text1")
-        r1()
-      })
-      output$text2 <- renderText({
-        pushMessage("output$text2")
-        input$go
-        r2()
-      })
-    })
-  }
-
-  testServer(module, {
-    expect_length(messages, 0)
-    session$setInputs(go = 1)
-    expect_equal(output$text1, "r1 1")
-    expect_equal(output$text2, "r2 1")
-    expect_equal(messages, list(
-      "output$text1",
-      "promise 1",
-      "output$text2",
-      "promise 2",
-      "promise 1 then 1",
-      "promise 2 then 1"
-    ))
-    clearMessages()
-    session$setInputs(go = 2)
-    expect_equal(output$text1, "r1 2")
-    expect_equal(output$text2, "r2 2")
-    expect_equal(messages, list(
-      "output$text1",
-      "promise 1",
-      "output$text2",
-      "promise 2",
-      "promise 1 then 2",
-      "promise 2 then 2"
-    ))
-  })
+test_that("isServer is only returns true for server funtions", {
+  expect_false(isServer(10))
+  expect_false(isServer(function(x) {}))
+  expect_false(isServer(function(output, session, input) {}))
+  expect_true(isServer(function(input, output, session) {}))
 })
